#!/usr/bin/env python

# following steps in Create a beautiful Space scene with Gimp 2.10 tutorial by conbagui found at link:http://fav.me/dd29p90
# It Creates a 1920 x 1080 px Universe Stars, Planet.
# It have the option for a Star flare

# author: Pocholo
# date: 5/13/20

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user script folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.



from gimpfu import *

def space_scape(imgSize, sunLayer):

	widthsize = 1920
	heightsize = 1080
	
	
	pdb.gimp_context_push()
	
	#Create the star layer
	img = pdb.gimp_image_new(widthsize, heightsize, RGB)
	starsLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Stars", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, starsLayer, 0)
	pdb.gimp_context_set_foreground((000, 000, 000))
	pdb.gimp_drawable_fill(starsLayer, FILL_FOREGROUND)

	
	#Create the Far stars
	pdb.plug_in_hsv_noise(img, starsLayer, 8, 0, 0, 255)
	pdb.gimp_brightness_contrast(starsLayer, -80, 50)
	pdb.plug_in_gauss(img, starsLayer, 0.50, 0.50, 1)
	pdb.plug_in_sparkle(img, starsLayer, 0.001, 1.00, 7, 4, 15, 1.00, 0, 0, 0, FALSE, FALSE, FALSE, 0)
	non_empty = pdb.gimp_edit_copy(starsLayer)
	 
	#Create the near stars
	nearLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Near stars", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, nearLayer, 0)
	pdb.gimp_layer_add_alpha(nearLayer)
	pdb.gimp_drawable_fill(nearLayer, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(nearLayer, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	pdb.gimp_layer_scale(nearLayer, 3000, 1688, FALSE)
	pdb.gimp_layer_resize_to_image_size(nearLayer)
	pdb.gimp_layer_set_mode(nearLayer, LAYER_MODE_SCREEN)
	nearLayer = pdb.gimp_image_merge_down(img, nearLayer, CLIP_TO_IMAGE)
	non_empty = pdb.gimp_edit_copy(nearLayer)
	
	#Create the near stars
	nearestLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Nearest stars", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, nearestLayer, 0)
	pdb.gimp_layer_add_alpha(nearestLayer)
	pdb.gimp_drawable_fill(nearestLayer, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(nearestLayer, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	pdb.gimp_layer_scale(nearestLayer, 4000, 2250, FALSE)
	pdb.gimp_layer_resize_to_image_size(nearestLayer)
	pdb.gimp_layer_set_mode(nearestLayer, LAYER_MODE_SCREEN)
	nearestLayer = pdb.gimp_image_merge_down(img, nearestLayer, CLIP_TO_IMAGE)
	
	#Create the space nebula
	
	#Create the nebula color
	colorLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Color", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, colorLayer, 0)
	pdb.gimp_layer_add_alpha(colorLayer)
	pdb.gimp_drawable_fill(colorLayer, FILL_TRANSPARENT)
	pdb.plug_in_plasma(img, colorLayer, 156, 1)
	pdb.gimp_layer_set_opacity(colorLayer, 2)
	
	#create the nebula layer
	nebulaLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Nebula", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, nebulaLayer, 0)
	pdb.gimp_layer_add_alpha(nebulaLayer)
	pdb.gimp_drawable_fill(nebulaLayer, FILL_TRANSPARENT)
	pdb.plug_in_solid_noise(img, nebulaLayer, FALSE, FALSE, 20, 15, 5, 5)
	pdb.gimp_layer_set_mode(nebulaLayer, LAYER_MODE_DODGE)
	
	
	#Create the planet and the planet shadow
	
	#Create the planet
	planetLayer = pdb.gimp_layer_new(img, 600, 600, RGB, "Planet", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, planetLayer, 0)
	pdb.gimp_layer_add_alpha(planetLayer)
	pdb.gimp_drawable_fill(planetLayer, FILL_TRANSPARENT)
	pdb.gimp_layer_translate(planetLayer, 660, 240)
	pdb.plug_in_plasma(img, planetLayer, 156, 1)
	pdb.plug_in_emboss(img, planetLayer, 90, 45, 3, FALSE)
	pdb.plug_in_applylens(img, planetLayer, 1.5, FALSE, FALSE, TRUE)
	pdb.gimp_layer_resize_to_image_size(planetLayer)
		
	#Create the planet shadow
	shadowLayer = pdb.gimp_layer_new(img, widthsize, heightsize,  RGB, "Planet shadow", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, shadowLayer, 0)
	pdb.gimp_layer_add_alpha(shadowLayer)
	pdb.gimp_drawable_fill(shadowLayer, FILL_TRANSPARENT)
	pdb.gimp_context_set_brush("2. Hardness 100")
	pdb.gimp_context_set_brush_size(1000)
	pdb.gimp_context_set_brush_aspect_ratio(-0.60)
	pdb.gimp_paintbrush_default(shadowLayer, 2, [960, 540])
	pdb.plug_in_gauss(img, shadowLayer, 180, 180, 1)
	pdb.gimp_image_set_active_layer(img, planetLayer)
	pdb.gimp_image_select_item(img, 2, planetLayer)
	pdb.gimp_selection_invert(img)
	pdb.gimp_image_set_active_layer(img, shadowLayer)
	pdb.gimp_edit_clear(shadowLayer)
	
	#Create the planet shine rim
	
	#Create the planet shine
	pdb.gimp_selection_invert(img)
	shineLayer = pdb.gimp_layer_new(img, widthsize, heightsize,  RGB, "Planet shine", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, shineLayer, 1)
	pdb.gimp_layer_add_alpha(shineLayer)
	pdb.gimp_drawable_fill(shineLayer, FILL_TRANSPARENT)
	pdb.gimp_drawable_edit_fill(shineLayer, FILL_WHITE)
	pdb.gimp_context_set_brush("2. Hardness 100")
	pdb.gimp_context_set_brush_size(615)
	pdb.gimp_context_set_brush_aspect_ratio(-0.05)
	pdb.gimp_paintbrush_default(shineLayer, 2, [960, 540])
	pdb.gimp_eraser_default(shineLayer, 2, [960, 540])
	pdb.plug_in_gauss(img, shineLayer, 21, 21, 1)
	pdb.gimp_layer_set_mode(shineLayer, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_selection_none(img)
	
	#Duplicate the planet layer for shinner rim
	pdb.gimp_image_set_active_layer(img, planetLayer)
	non_empty = pdb.gimp_edit_copy(planetLayer)
	planetcopy = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Planet copy", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, planetcopy, -1)
	pdb.gimp_layer_add_alpha(planetcopy)
	pdb.gimp_drawable_fill(planetcopy, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(planetcopy, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	pdb.gimp_layer_set_mode(planetcopy, LAYER_MODE_SOFTLIGHT)
	planetcopy = pdb.gimp_image_merge_down(img, planetcopy, CLIP_TO_IMAGE)
	
	#Create brightness on the background stars an nebula
	pdb.gimp_image_set_active_layer(img, shadowLayer)
	beautyLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Beauty", 100, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_image_add_layer(img, beautyLayer, 0)
	pdb.gimp_context_set_foreground((000, 000, 000))
	pdb.gimp_drawable_fill(beautyLayer, FILL_FOREGROUND)
	pdb.plug_in_nova(img, beautyLayer, 960, 540, (000, 000, 255), 300, 1, 0)
	
	
	#Create a farther big star
	
	#create the big star layer
	pdb.gimp_image_set_active_layer(img, nebulaLayer)
	bigStar = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Big Star", 100, LAYER_MODE_SOFTLIGHT)
	pdb.gimp_image_add_layer(img, bigStar, 4)
	pdb.gimp_layer_add_alpha(bigStar)
	pdb.gimp_drawable_fill(bigStar, FILL_TRANSPARENT)
	
	#create the big star
	
	name = pdb.gimp_context_get_brush()
	pdb.gimp_context_set_brush ("2. Block 03")
	pdb.gimp_context_set_brush_size(200)
	pdb.gimp_context_set_brush_aspect_ratio(10)
	pdb.gimp_context_set_brush_angle(0)
	pdb.gimp_context_set_foreground ((255, 255, 255))
	pdb.gimp_paintbrush_default(bigStar, 2, [1675, 525])
	pdb.plug_in_gauss(img, bigStar, 300, 0, 1)
	pdb.plug_in_nova(img, bigStar, 1675, 525, (100, 46, 57), 25, 25, 0)
	pdb.gimp_layer_set_mode(bigStar, LAYER_MODE_SCREEN)
	
	#Make the big star brighter
	non_empty = pdb.gimp_edit_copy(bigStar)
	brightnessLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Brightness", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, brightnessLayer, -1)
	pdb.gimp_layer_add_alpha(brightnessLayer)
	pdb.gimp_drawable_fill(brightnessLayer, FILL_TRANSPARENT)
	floating_sel = pdb.gimp_edit_paste(brightnessLayer, FALSE)
	pdb.gimp_floating_sel_anchor(floating_sel)
	pdb.gimp_layer_set_mode(brightnessLayer, LAYER_MODE_SCREEN)
	
			
	#Create the sun flare
	if sunLayer == True:
		sunLayer = pdb.gimp_layer_new(img, widthsize, heightsize, RGB, "Sun and flare", 100, LAYER_MODE_SCREEN)
		pdb.gimp_image_add_layer(img, sunLayer, 1)
		pdb.gimp_context_set_foreground((000, 000, 000))
		pdb.gimp_drawable_fill(sunLayer, FILL_FOREGROUND)
		pdb.plug_in_flarefx(img, sunLayer, 1250, 500)


	gimp.Display(img)
	
	pdb.gimp_context_pop()


register(
	"space_scape",
	"Creates stars and a planet",
	"Creates stars and a planet",
	"Pocholo",
	"Pocholo",
	"2020",
	"Stars and planet",
	"",
[
	(PF_ADJUSTMENT, "imgSize", "Image size", 1920, (1920, 1920, 1)),
	(PF_BOOL, "sunLayer", "Sun Flare?", TRUE),
],
 [],
space_scape, menu="<Image>/Python-Fu/Spacescape")

main()	

